'use strict';

exports.__esModule = true;

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

exports.compile = compile;
exports.compileFile = compileFile;
exports.compileByBabel = compileByBabel;
exports.getFiles = getFiles;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var fs = require('fs');
var path = require('path');
var fse = require('fs-extra');
var fileUtil = require('./file');

//babel not export default property
//so can not use `import babel from 'babel-core'`
var babel = require('babel-core');

var allowFileExt = ['.js'];

/**
 * store compiled files last mtime
 * @type {Object}
 */
var compiledMtime = {};

/**
 * compiled error files
 * @type {Array}
 */
var compiledErrorFiles = [];

/**
 * compile files
 * @return {} []
 */
function compile(srcPath, outPath) {
  var files = getFiles(srcPath, true);

  var changedFiles = [];

  console.log(files);

  files.forEach(function (file) {
    var srcFullPath = path.join(srcPath, file);
    var saveOutFullPathpath = path.join(outPath, file);

    var extname = path.extname(file);

    //if is not js file, only copy
    if (allowFileExt.indexOf(extname) === -1) {
      compileFile(srcFullPath, saveOutFullPathpath, true);
      return;
    }

    var mTime = fs.statSync(srcFullPath).mtime.getTime();

    if (fileUtil.isFile(saveOutFullPathpath)) {
      var outmTime = fs.statSync(saveOutFullPathpath).mtime.getTime();

      // if compiled file mtime is later than source file,
      // it means source file not modified, so there is no necessary to compile.
      if (outmTime >= mTime) {
        return;
      }
    }

    if (!compiledMtime[file] || mTime > compiledMtime[file]) {
      var ret = compileFile(srcFullPath, saveOutFullPathpath);

      if (ret) {
        changedFiles.push(saveOutFullPathpath);
      }

      compiledMtime[file] = mTime;

      var index = compiledErrorFiles.indexOf(file);

      if (ret) {
        if (index > -1) {
          compiledErrorFiles.splice(index, 1);
        }
      } else if (ret === false) {
        if (index === -1) {
          compiledErrorFiles.push(file);
        }
      }
    }
  });

  // console.error(compiledErrorFiles)
}

/**
 * compile single file
 * @param  {String} file     []
 * @param  {Boolean} [onlyCopy] []
 * @return {}          []
 */
function compileFile(srcFullPath, saveOutFullPath, onlyCopy) {
  var content = fs.readFileSync(srcFullPath, 'utf8');

  //when get file content empty, maybe file is locked
  if (!content) {
    return;
  }

  // only copy file content
  if (onlyCopy) {
    fse.copySync(srcFullPath, saveOutFullPath);
    return;
  }

  try {
    var startTime = Date.now();

    var data = compileByBabel(content, {
      filename: srcFullPath
      // sourceMaps: true,
      // sourceFileName: relativePath
    });

    var endTime = Date.now();
    console.log('Compile file ' + srcFullPath, 'Babel cost ' + (endTime - startTime));

    // save file
    fse.outputFileSync(saveOutFullPath, data.code);

    return true;
  } catch (e) {
    console.error('compile file ' + srcFullPath + ' error', e);
  }

  return false;
}

/**
 * babel compile
 * https://babeljs.io/docs/core-packages/#babeltransformcode-string-options-object
 * @return {} []
 */
function compileByBabel(content, options) {
  return babel.transform(content, (0, _assign2.default)({
    presets: ['es2015-loose', 'stage-1'],
    plugins: ['transform-runtime']
  }, options));
}

/**
 * get all files
 * @param paths
 */
function getFiles(paths) {
  var result = fileUtil.getAllFiles(paths);

  var files = result.map(function (item) {
    return item.relativePath;
    // return path.join(item.basePath, item.relativePath);
  });

  return files;
}
//# sourceMappingURL=babel-compile.js.map